#blank

import matplotlib, sys, re
from optparse import OptionParser
parser = OptionParser()
parser.add_option("-i","--interactive", action='store_true',
                  dest = 'interactive', default = False,
                  help="Interactive plotting on OSX")
parser.add_option("-f", "--fileroot", dest="root", default = 'source',
                  help="Root for various file name")
parser.add_option("-s", "--source", dest="source", default = 'all',
                  help="Set source name or 'all'")
parser.add_option("-c", "--checkpoint", action='store',
                  dest='checkpoint', type = 'int', default = 0,
                  help = "Checkpoint start of flow (0=entire program)" )
parser.add_option("-t", "--test", action='store_true',
                  dest='test', default = False,
                  help = "Work on test data" )
parser.add_option("-n", "--notest", action='store_false',
                  dest='test', default = False,
                  help = "Not work on test data" )
parser.add_option("-o", "--onetime", action='store_true',
                  dest='short', default = False,
                  help = "Do a single interval" )
parser.add_option("-a", "--aipsfrmap", action='store_true',
                  dest='usefrmap', default = False,
                  help = "Use AIPS FRMAP for measuring fringe rates" )
parser.add_option("-r", "--red_peak", action='store_false',
                  dest='doblue', default = True,
                  help = "Do second (red-shifted) peak" )
parser.add_option("-y", "--array", action='store',
                  dest='array', default = 'all',
                  help = "Array to consider, 'all' or 2-letter codes: McWb")
parser.add_option("-w", "--sol_window", action = 'store',
                  dest='window', type = "float", default= 3,
                  help = "Time window for determining Fringe Rate solutions") 

(options, args) = parser.parse_args()
print 'options:',options

if options.test and options.root == 'source':
    options.root = 'test'

interactive = options.interactive

if options.interactive:
    matplotlib.use('MacOSX')
else:
    matplotlib.use('pdf')


from AIPS import AIPS, AIPSDisk
from AIPSData import AIPSUVData, AIPSImage
from Wizardry.AIPSData import AIPSUVData as WAIPSUVData
from AIPSTask import AIPSTask, AIPSList

import pickle

from Ea38DataClass import *
from HuibFrmapTasks import *
from FrmapUtils import *


#------------------------------------------------------------
#
#   Start main program
#   Script to ensure accountability, not automation!
#   Calibration of EA38, recorrelatio, line pass, avg 2s, 16ch

AIPS.userno = 1522
ea38 = ProjectPars()
docal = 3
if options.test: 
    docal = 0

if options.source == 'all':
    iterkeys = ea38.src.keys()
else:
    iterkeys = [options.source]
print 'iter',iterkeys
if options.test: iterkeys = ['3c3']

isrc = 0

for short in iterkeys:
    isrc = isrc +1

    source = ea38.src[short].name
    root = options.root+short
    frlog = open( root+'_out.txt','w')
    print >> frlog, 'options:', options
    print >> frlog, 'iter',iterkeys
    print 'doing:',short, ea38.src[short].name
    print >> frlog, 'doing:',short, ea38.src[short].name

    if options.test:
        print 'in test'
        set = ea38.test
        print set.name
    else:
        if ea38.src[short].cont():
            set = ea38.cont
        else:
            set = ea38.line
    print set.name, set.clas
    uvdata = AIPSUVData( set.name, set.clas, set.disk, set.seq)
    wuvdata = WAIPSUVData( set.name, set.clas, set.disk, set.seq)

    if ea38.src[short].cont():
        bchan = 1
        echan = 0
    else:
        if options.doblue:
            bchan = ea38.src[short].ib
            echan = ea38.src[short].eb
        else:
            bchan = ea38.src[short].ir
            echan = ea38.src[short].er
    if not bchan:
        print 'No channel selection'
        continue

    if uvdata.exists():
        print 'OK let\'s start'
    else:
        raise RuntimeError, "Files not OK"

    andict = antennalist( uvdata)
    selectan = anselect( andict, options.array )
    print selectan,annames( andict, selectan )
    print >> frlog, 'antennas:', selectan,annames( andict, selectan )

    solint = options.window
    timers = timeranges(uvdata,source,solint)
    if options.short:
        timers = [ timers[0] ]
        if options.test:
            timers = [ [ 0,20,0,0,0,20,3,0] ]
            #use KnEfOnTr for quick tests

    window = 10
    lines = []
    stats = Statistics(andict)

    pickname = root+'_pickle.dat'

    # check point 0 get coordinates and fringe rates
    if options.checkpoint < 1:
        reduvdt(wuvdata,timers,selectan,lines, frlog )

    #Now the fringe rates 
        for itime in range(len(timers)):
            stats.count('times')
            print '***Start %s - %s' % (stime(timers[itime][0:4]),
                                        stime(timers[itime][4:8]))
            print >> frlog, ('\nStart %s - %s' % 
                             (stime(timers[itime][0:4]),
                              stime(timers[itime][4:8]) ))
        #fullfrmap(uvdata,source,timers[k],bchan,echan,
        #selectan,docal, solint, frlog)

            for ilin in range(len(lines[itime])):
                iant = lines[itime][ilin]['basl'][0]
                jant = lines[itime][ilin]['basl'][1]
                stats.count('baslints')
                print '***Start %i,%i %s - %s' % (iant,
                                                  jant, stime(timers[itime][0:4]),
                                                  stime(timers[itime][4:8]))
                print '   Progres: sour:%i/%i tim:%i/%i basl%i/%i' % ( isrc,
                                                       len(iterkeys), 
                                                       itime, len(timers), 
                                                       ilin, len(lines[itime]) ) 

                if options.usefrmap:
                    (tmp1, tmp2, fr, dfr) = logfrmap(uvdata,
                                source,timers[itime],bchan,echan,
                                iant,jant,docal,solint,frlog)
                else:
                    #(dudt, dvdt) = duvdt(wuvdata,timers[itime],iant,jant,frlog)
                    #print 'now  :',itime, iant, jant, dudt, dvdt 
                    #print 'ready:',itime, lines[itime][ilin]
                    (fr, dfr) = logbling(uvdata,source,timers[itime],
                                         bchan,echan,iant,jant,docal,
                                         solint,frlog)
                
                stats.countbasltot(iant,jant)
                if (fr and dfr):
                    stats.count('lines')
                    stats.countbaslok(iant,jant)

                
                lines[itime][ilin].update({'fr':fr, 'dfr':dfr })

        #print >> frlog, lines
        #print '\n resuts in\n', lines
        pickf = open(pickname, 'w')
        pickle.dump( lines, pickf ) 
        pickle.dump( stats, pickf )
        pickf.close()

    if options.checkpoint < 2:
        picki = open(pickname, 'r') 
        lines = pickle.load( picki )
        stats = pickle.load( picki )
        plotuvdetect( lines, andict, root, options.interactive )


    if options.checkpoint < 3:
        picki = open(pickname, 'r') 
        lines = pickle.load( picki )
        stats = pickle.load( picki )
        plotfrmap(window,lines,andict,root, options.interactive)

    if options.checkpoint < 4:

        picki = open(pickname, 'r') 
        lines = pickle.load( picki )
        stats = pickle.load( picki )

        tripos = []
        for t in range(len(lines)):
            for i in range(len(selectan)):
                for j in range(i+1,len(selectan)):
                    for k in range(j+1,len(selectan)):
                        iant = selectan[i];jant = selectan[j]
                        kant = selectan[k]
                        trifit = triasect( lines[t], iant, jant, kant )
                        #print 'Result', iant, jant, kant, stats.triangle
                        stats.count('triangle')
                        if trifit['nsol'] > 0:
                            stats.count('triaok')
                            if trifit['nsol'] > 2: stats.count('triatri')
                            tripos.append( trifit )
                                             
        print '\nStats:\n', stats
        print >> frlog, stats
        (posra, posdec, sqra, sqdec ) = plotfrpos(window, tripos, andict,
                                                  root, 
                                                  options.interactive)
        print posra, posdec, sqra, sqdec
        print 'avg position %5.3f +/- %5.3f, %5.3f +/- %5.3f' % (posra,sqra,
                                                         posdec,sqdec)
        print >> frlog, ( 'avg position: %5.3f +/- %5.3f, %5.3f +/- %5.3f' % (posra,sqra,
                                                         posdec,sqdec))



        (posra, posdec, sqra, sqdec ) = plotfrwgh(window, tripos, andict,
                                                  root, 
                                                  options.interactive)
        print 'weighted pos: %5.3f +/- %5.3f, %5.3f +/- %5.3f' % (posra,sqra,
                                                         posdec,sqdec)
        print >> frlog, ('weighted pos %5.3f +/- %5.3f, %5.3f +/- %5.3f' % (posra,sqra,
                                                         posdec,sqdec) )
    
    #skip for now
    if options.checkpoint < -1:

        picki = open(pickname, 'r') 
        lines = pickle.load( picki )
        stats = pickle.load( picki )        

        ploterdist( lines, andict, root, options.interactive )


    print 'Finished ', source

print 'Reached bottom'
