
from AIPSTask import AIPSTask, AIPSList

def stime(atime):
    #print atime
    stime = '%id%ih%im%.2f' % (atime[0],atime[1],atime[2],atime[3])
    return stime

def antennalist( dataset ):
    antab = dataset.table('AN',1)

    andict = {}
    for row in antab:
        twocode = row.anname.split()[0]
        twocode = twocode[0]+twocode[1:].lower()
        #print row.nosta, twocode,'<'
        andict.update({ row.nosta: twocode})

    return andict

def timeranges(dataset, source, window):
    import math

    def time4(time):
        day = int(time)
        left = 24 * ( time - day )
        hour = int(left)
        left = 60 * (left - hour)
        min = int(left)
        sec = 60 * (left - min)
        return day, hour, min, sec

    su_index = -1
    sourcelist = dataset.table('SU',1)
    for row in sourcelist:
        dummy = 1
        if row.source.find(source) == 0:
            #print 'yes',row.source,row.id__no
            su_index = row.id__no
    #print 'check: ',source,dataset.sources[su_index-1]
            
    index = dataset.table('NX',1)
    windowsday = window/(24.*60)
    timers = []
    for row in index:
        if row.source_id == su_index:
            if row.time_interval < windowsday:
            #print time4(row.time), time4(row.time_interval)
                thisrange = [8]
                thisrange[0:3] = time4(row.time-0.5*row.time_interval)
                thisrange[4:7] = time4(row.time+0.5*row.time_interval)
                #print 'EASY', thisrange
                timers.append(thisrange)
            else:
                #print 'HARD', time4(row.time-0.5*row.time_interval), time4(row.time+0.5*row.time_interval)
                ndiv = int(math.ceil(row.time_interval/windowsday))
                newint = (row.time_interval/ndiv)
                #print '   ', ndiv, newint*60*24
                start = row.time-0.5*row.time_interval
                for inew in range(ndiv):
                    thisrange = [8]
                    thisrange[0:3] = time4(start)
                    start=start+newint
                    thisrange[4:7] = time4(start)
                    timers.append(thisrange)
                    #print 'FIXD', thisrange

    return timers


def logbling(uvdata,source,timers,bchan,echan,anta,antb,docal,solint,log):

    uvdata.zap_table('AIPS BS', -1)
    #log.write('BLING %i,%i %s - %s \n' % (anta,antb,
    #                                      stime(timers[0:4]),
    #                                      stime(timers[4:8])) )
    bling = AIPSTask('bling')

    bling.indata = uvdata
    bling.timerang = AIPSList(timers)
    #bling.timerang = AIPSList([1,5,7,1,1,5,10,0])
    bling.stokes = 'll'
    bling.bchan = bchan
    bling.echan = echan
    bling.calsour = AIPSList([source])
    bling.antennas = AIPSList([anta])
    bling.baseline = AIPSList([antb])
    bling.dparm = AIPSList([0,-1,0,-1,0,200,0])
    if docal:
        bling.docalib = 1
        bling.gainuse = docal
        if bchan == 1 or echan ==1 or echan == 0:
            bling.doband = 0
            bling.bpver = 0            
        else:
            bling.doband = 1
            bling.bpver = 1
    else:
        bling.docalib = -1
        bling.gainuse = 0
        bling.doband = 0
        bling.bpver = 0

    bling.solint = solint
    bling.opcode = 'RATE'
    #bling.inputs()
    try:
        bling()
    except:
        uvdata.clrstat()
        print '*BLING seemed to have failed'
        return ( None, None )

    blinglist = uvdata.table('BS',1)
    fr = None; dfr = None
    irow = 0
    for row in blinglist:
        #print row
        irow = irow +1
        log.write('BLING %i,%i %f %f\n' % (row.baseline[0], 
                                         row.baseline[1], 
                                         row.residual_fringe_rate, 
                                         row.fringe_rate_error))
        fr = row.residual_fringe_rate
        sfr = row.fringe_rate_error

    if irow == 1:
        return (fr, sfr)
    else:
        print "BLING not a single solution"
        return (None, None)

def logfrmap(uvdata,source,timer,bchan,echan,iant,jant,docal,solint,log ):
    import re
    log.write('FRMAP %i,%i %s - %s \n' % (iant,jant,
                                          stime(timer[0:4]),
                                          stime(timer[4:8])) )
    frmap = AIPSTask('frmap')
    tmp = 'frmap.tmp'
    frmap.log = open(tmp,'w')

    frmap.indata = uvdata
    frmap.sources = AIPSList([source])
    frmap.timerang = AIPSList(timer)
    frmap.bchan = bchan
    frmap.echan = echan
    frmap.channel = -1
    frmap.antennas = AIPSList([iant])
    frmap.baseline = AIPSList([jant])
    if docal:
        frmap.docalib = 1
        frmap.gainuse = docal
        if bchan == 1 or echan ==1 or echan == 0:
            frmap.doband = 0
            frmap.bpver = 0            
        else:
            frmap.doband = 1
            frmap.bpver = 1
    else:
        frmap.docalib = -1
        frmap.gainuse = 0
        frmap.doband = 0
        frmap.bpver = 0
    if echan == 0:
        avgchan = 16
    else:
        avgchan = bchan - echan
    
    frmap.aparm = AIPSList([2,0,avgchan,0,0,10000,10000,0,0,1])
    frmap.bparm = AIPSList([0,0,solint,0,0,0,0,0,2,0])
    frmap.dotv = -1
    frmap.grchan = 2
    #frmap.inputs()
    try:
        frmap()
    except:
        uvdata.clrstat()
        print '*FRMAP seemed to have failed'

    frmap.log.close()
    
    back = open( tmp, 'r')
    ( xcoeff, ycoeff, rate, drate  ) = None, None, None, None
    ifr = 0
    for line in back:
        yesrate = re.match(r"^FRMAP\d\:\s*(\S*)\s\*\s+X\s+\+\s+(\S*)\s+\*\s+Y\s+\=\s+(\S*)\+\-(\S*)\s+mHZ\;\s*(\S*)\s*mJy$", 
                           line)
        
        if yesrate:
            ifr = ifr + 1
            xcoef = float(yesrate.group(1))
            ycoef = float(yesrate.group(2))
            rate = float(yesrate.group(3))
            drate = float(yesrate.group(4))
            flux = float(yesrate.group(5))
            #print 'coeff_x:',xcoef,'coeff_y',ycoef,'rate',rate,'drate:',drate,'flux:',flux
            log.write('FRMAP coeff_x: % f coeff_y: % f\n' % 
                      (xcoef,ycoef))
            log.write('FRMAP rate: % f drate: %f flux: %f\n' % 
                      (rate,drate,flux))
    if ifr == 1:
        return (xcoef, ycoef, rate/1e3, drate/1e3)
    else:
        return (None, None, None, None )

def reduvdt(uvdata,timers,selectan, lines,log):
    import math
        
    log.write('DUVDT entire set \n' )
    print 'DUVDT entire set \n'

    def timef(time):
        #print time
        timef=time[0]+(time[1]+(time[2]+time[3]/60.)/60.)/24.
        return timef

    def radpas( rad ):
        radpas = ( rad*math.pi )/(180.*3600)
        return radpas
    def deg2rad( deg ):
        deg2rad = ( deg*math.pi )/(180.)
        return deg2rad

    def ininterval( vistim, timers ):

        for it in range(len(timers)):
            if ( vistim > timef(timers[it][0:4]) and
                 vistim < timef(timers[it][4:8])):
                kt = it
                return ( True, kt )
        return (False, None)


    uvfound = {}
    for visibility in uvdata:
        if visibility.baseline[1] > visibility.baseline[0]:
            iant = visibility.baseline[0]
            jant = visibility.baseline[1]
        elif visibility.baseline[1] < visibility.baseline[0]:
            iant = visibility.baseline[1]
            jant = visibility.baseline[0]
        else:
            raise 'autocorelations!'
        #print iant, jant, timef(timers[0:4]), timef(timers[4:8])
        #print visibility.uvw, visibility.time, visibility.baseline
        if ( iant in selectan and 
             jant in selectan ):
            ( inin, kt ) = ininterval(visibility.time, timers ) 
            if inin:
                uvkey = 10000 * kt + 100 * jant + iant
                if uvfound.has_key( uvkey ):
                    uvfound[uvkey].update( {'umax': visibility.uvw[0],
                                            'vmax': visibility.uvw[1],
                                            'tmax': visibility.time } )
                else: 
                    uvfound.update( { uvkey: { 'umin': visibility.uvw[0],
                                               'vmin': visibility.uvw[1],
                                               'tmin': visibility.time } } )
                #print uvfound

    for itime in range(len(timers)):
        lines.append( [] )
        #print 'test',itime,':',lines
        for i in range(len(selectan)):
            for j in range(i+1,len(selectan)):
                iant = selectan[i];jant = selectan[j]

                uvkey = 10000 * itime + 100 * jant + iant
                #print itime, uvkey, uvfound.keys()
                if uvfound.has_key(uvkey):
                    if uvfound[uvkey].has_key('tmax'):
                        deltat = (uvfound[uvkey]['tmax']-
                                  uvfound[uvkey]['tmin'])*24.*60.*60.
                        umax = uvfound[uvkey]['umax']
                        umin = uvfound[uvkey]['umin']
                        vmax = uvfound[uvkey]['vmax']
                        vmin = uvfound[uvkey]['vmin']
                        dudt = (umax-umin)/deltat
                        dvdt = (vmax-vmin)/deltat
                
                        baseline = [iant, jant ]

                        lines[itime].append({'a': radpas(dudt), 
                                             'b': radpas(dvdt),
                                             'basl': baseline,
                                             'u1': umin/1000.,
                                             'u2': umax/1000.,
                                             'v1': vmin/1000.,
                                             'v2': vmax/1000. })
                    else:
                        print 'no',itime
                        lines[itime].append({'a': None, 
                                             'b': None,
                                             'basl': baseline,
                                             'u1': None,
                                             'u2': None,
                                             'v1': None,
                                             'v2': None })

                    log.write('DUVDT %i,%i,%i dT = %-5.2f dudt = %05.2f dvdt = %6.1f\n' % 
                              (itime, iant, jant, deltat,dudt,dvdt))
    return

def duvdt(uvdata, timers,iant,jant,log):
    import math
        
    log.write('DUVDT %i,%i %s - %s \n' % (iant,jant,stime(timers[0:4]),
                                          stime(timers[4:8])))
    print 'DUVDT %i,%i %s - %s' % (iant,jant,stime(timers[0:4]),
                                          stime(timers[4:8]))
    def timef(time):
        #print time
        timef=time[0]+(time[1]+(time[2]+time[3]/60.)/60.)/24.
        return timef

    def radpas( rad ):
        radpas = ( rad*math.pi )/(180.*3600)
        return radpas
    def deg2rad( deg ):
        deg2rad = ( deg*math.pi )/(180.)
        return deg2rad

    umin = 0
    umax = 0
    vmin = 0
    vmax = 0
    tmin = 0
    tmax = 0
    start = 1
    icount = 0
    for visibility in uvdata:
        #print iant, jant, timef(timers[0:4]), timef(timers[4:8])
        #print visibility.uvw, visibility.time, visibility.baseline
        if ( (iant == visibility.baseline[0] or 
              iant == visibility.baseline[1]) and 
             (jant == visibility.baseline[0] or 
              jant == visibility.baseline[1]) ):
            if (visibility.time > timef(timers[0:4]) and 
                visibility.time < timef(timers[4:8]) ):
                icount = icount + 1
                if start:
                    umin = visibility.uvw[0]
                    vmin = visibility.uvw[1]
                    tmin = visibility.time
                    start = 0
                else:
                    umax = visibility.uvw[0]
                    vmax = visibility.uvw[1]
                    tmax = visibility.time
                    
    if icount > 0:
        deltat = (tmax-tmin)*24.*60.*60.
        dudt = (umax-umin)/deltat
        dvdt = (vmax-vmin)/deltat
        log.write('DUVDT dT = %-5.2f dudt = %05.2f dvdt = %6.1f\n' % 
                  (deltat,dudt,dvdt))
        #tmp = radpas(dudt)*math.cos(deg2rad(39.76))
        #log.write('DUVDT dudt = %8.6f (%8.6f)[as] dvdt = %8.6f [as]\n' % 
        #          (radpas(dudt),tmp,radpas(dvdt) ))
        log.write('DUVDT dudt = %8.6f [as] dvdt = %8.6f [as]\n' % 
         
         (radpas(dudt),radpas(dvdt) ))
        return ( radpas(dudt), radpas(dvdt) )
    else:
        log.write('DUVDT no data %i,%i %s - %s \n' % (iant,
                                          jant,stime(timers[0:4]),
                                          stime(timers[4:8])))
        return ( None, None)
